function [caldata prbdata] = calprbdata(path, depth, masks, links)
% --- Gathers data on calibrations & probes

% ---------
% --- Notes
% ---------

% path == system path
% depth == find_system parameter SearchDepth (0, 1, 2, etc.)
% masks == find_system parameter LookUnderMasks ('none', 'all', etc.)
% links == find_system parameter FollowLinks ('on' or 'off')

% ------------------------------
% --- Find Calibrations & Probes
% ------------------------------

% use ReferenceBlock for lookup tables bacause of S-functions in motohawk_table_1d and motohawk_table_2d
tblblklst = {'MotoHawk_lib/Lookup Tables/motohawk_prelookup'; ...
    'MotoHawk_lib/Lookup Tables/motohawk_interpolation_1d';
    'MotoHawk_lib/Lookup Tables/motohawk_interpolation_2d';...
    'MotoHawk_lib/Lookup Tables/motohawk_table_1d';
    'MotoHawk_lib/Lookup Tables/motohawk_table_2d'; ...
    'MotoHawk_lib/Fixed Point Blocks/Fixed Point Prelookup'; ...
    'MotoHawk_lib/Fixed Point Blocks/Fixed Point Interpolation 1D';
    'MotoHawk_lib/Fixed Point Blocks/Fixed Point Interpolation 2D'};
callst = find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_calibration', 'storage', 'Calibration');
callst = [callst; find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_calibration', 'storage', 'Calibration NV')];
for i = 1:length(tblblklst)
    callst = [callst; find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'ReferenceBlock', tblblklst{i})];
end
prblst = find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_probe');
prblst = [prblst; find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_calibration', 'storage', 'Display')];
prblst = [prblst; find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_calibration', 'storage', 'Display NV')];

% add data def blocks and structure vardec blocks to lists
datadefstructlst = find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_data_def', 'use_vardec', 'on');
datadefstructlst = [datadefstructlst; find_system(path, 'SearchDepth', depth, 'LookUnderMasks', masks, 'FollowLinks', links, 'FunctionName', 'motohawk_sfun_struct_vardec')];
for i = 1:length(datadefstructlst)
    if strcmp(get_param(datadefstructlst{i}, 'window'), 'Calibration')
        callst = [callst; datadefstructlst(i)];
    elseif strcmp(get_param(datadefstructlst{i}, 'window'), 'Display')
        prblst = [prblst; datadefstructlst(i)];
    end
end

% -------------------------
% --- Names & Alphabetizing
% -------------------------

calnams = {};
for i = 1:length(callst)
    try
        if strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Lookup Tables/motohawk_prelookup') || ...
               strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Fixed Point Blocks/Fixed Point Prelookup') 
            calnams{i} = [motohawk_get_param_ws(callst{i}, 'nam') 'IdxArr'];
        elseif strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Lookup Tables/motohawk_interpolation_1d') || ...
                strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Lookup Tables/motohawk_table_1d') || ...
                strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Fixed Point Blocks/Fixed Point Interpolation 1D')
            calnams{i} = [motohawk_get_param_ws(callst{i}, 'nam') 'Tbl'];
        elseif strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Lookup Tables/motohawk_interpolation_2d') || ...
                strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Lookup Tables/motohawk_table_2d') || ...
                strcmp(get_param(callst{i}, 'ReferenceBlock'), 'MotoHawk_lib/Fixed Point Blocks/Fixed Point Interpolation 2D')
            calnams{i} = [motohawk_get_param_ws(callst{i}, 'nam') 'Map'];
        elseif strcmp(get_param(callst{i}, 'FunctionName'), 'motohawk_sfun_struct_vardec')
            calnams{i} = motohawk_get_param_ws(callst{i}, 'struct_nam');
        elseif strcmp(get_param(callst{i}, 'FunctionName'), 'motohawk_sfun_data_def') && ...
                strcmp(get_param(callst{i}, 'typ'), 'struct container')
            calnams{i} = motohawk_get_param_ws(callst{i}, 'container_nam');
        else
            calnams{i} = motohawk_get_param_ws(callst{i}, 'nam');
        end
    catch
        calnams{i} = motohawk_get_param_ws(callst{i}, 'nam');
    end
    if isempty(calnams{i})
        calnams{i} = 'Could Not Eval. (Update Reqd.)';
    end
end
prbnams = {};
for i = 1:length(prblst)
    if strcmp(get_param(prblst{i}, 'FunctionName'), 'motohawk_sfun_struct_vardec')
        prbnams{i} = motohawk_get_param_ws(prblst{i}, 'struct_nam');
    elseif strcmp(get_param(prblst{i}, 'FunctionName'), 'motohawk_sfun_data_def') && ...
        strcmp(get_param(prblst{i}, 'typ'), 'struct container')
        prbnams{i} = motohawk_get_param_ws(prblst{i}, 'container_nam');
    else
        prbnams{i} = motohawk_get_param_ws(prblst{i}, 'nam');
    end
    if isempty(prbnams{i})
        prbnams{i} = 'Could Not Eval. (Update Reqd.)';
    end
end
[calnams calidx] = sort(calnams);
[prbnams prbidx] = sort(prbnams);

% --------
% --- Data
% --------

for i = 1:2
    data = {};
    % 1 == calibration data; 2 == probe data
    if i == 1
        lst = callst;
        nams = calnams;
        idx = calidx;
        lngth = length(calnams);
    else
        lst = prblst;
        nams = prbnams;
        idx = prbidx;
        lngth = length(prbnams);
    end
    for j = 1:lngth
        blk = lst{idx(j)};
        
        % name
        data.nam(j) = nams(j);
        
        % units
        data.units{j} = 'N/A';
        try data.units{j} = motohawk_get_param_ws(blk, 'units'); end
        try data.units{j} = motohawk_get_param_ws(blk, 'table_units'); end
        try hasenum = strcmp(get_param(blk, 'view_as'), 'Enumeration'); catch hasenum = 0; end
        if hasenum
            data.units{j} = strtrim([data.units{j} ' (enum.)']);
        end
        
        % default value
        dfltval = [];
        try dfltval = motohawk_get_param_ws(blk, 'data'); end
        try dfltval = motohawk_get_param_ws(blk, 'val'); end
        try dfltval = motohawk_get_param_ws(blk, 'breakpoint_data'); end
        try dfltval = motohawk_get_param_ws(blk, 'table_data'); end
        if isempty(dfltval)
            data.dfltval{j} = '(inherited)';
        elseif iscellstr(dfltval)
            data.dfltval{j} = '(struct)';
        else
            sz = size(dfltval);
            if sz(1) > 1 % put matrix on a single line
                dfltval2 = num2str(dfltval(1,:));
                for k = 2:sz
                    dfltval2 = [dfltval2 ' ; ' num2str(dfltval(k,:))];
                end
                data.dfltval{j} = dfltval2;
            else
                data.dfltval{j} = num2str(dfltval);
            end
        end

        % size
        data.size{j} = 'N/A';
        if strcmp(data.dfltval{j}, '(inherited)')
            data.size{j} = '(inherited)';
        elseif ~strcmp(data.dfltval{j}, '(struct)')
            try
                sz = size(dfltval);
                data.size{j} = [num2str(sz(1)) ' x ' num2str(sz(2))];
            end
        end
        
        % data type
        data.datatyp{j} = '(inherited)';
        try data.datatyp{j} = get_param(blk, 'typ'); end
        try data.datatyp{j} = get_param(blk, 'data_type'); end
        try data.datatyp{j} = ['int16 (FP B' num2str(motohawk_get_param_ws(blk, 'bscale')) ')']; end
        if strncmpi(data.datatyp{j}, 'Inherit', 7) == 1
            data.datatyp{j} = '(inherited)';
        end
        
        % precision
        data.prec{j} = 'N/A';
        try data.prec{j} = motohawk_get_param_ws(blk, 'precision'); end
        try data.prec{j} = motohawk_get_param_ws(blk, 't_precision'); end
        if strcmp(data.prec{j}, '')
            if strcmp(data.datatyp{j}, '(inherited)')
                data.prec{j} = '(inherited)';
            elseif strcmp(data.datatyp{j}, 'single') || strcmp(data.datatyp{j}, 'double')
                data.prec{j} = '2';
            else
                data.prec{j} = '0';
            end
        end
        if ~isempty(strfind(data.prec{j}, '.'))
            data.prec{j} = strtok(fliplr(data.prec{j}), '.');            
        end
        
        % gain
        data.gain{j} = 'N/A';
        try data.gain{j} = num2str(motohawk_get_param_ws(blk, 'gain')); end
        try data.gain{j} = num2str(motohawk_get_param_ws(blk, 't_gain')); end

        % offset
        data.offset{j} = 'N/A';
        try data.offset{j} = num2str(motohawk_get_param_ws(blk, 'offset')); end
        try data.offset{j} = num2str(motohawk_get_param_ws(blk, 't_offset')); end
        
        % exponent
        data.exp{j} = 'N/A';
        try data.exp{j} = num2str(motohawk_get_param_ws(blk, 'exp')); end
        try data.exp{j} = num2str(motohawk_get_param_ws(blk, 't_exp')); end
        
        % minimum
        data.min{j} = 'N/A';
        try data.min{j} = num2str(motohawk_get_param_ws(blk, 'min_val')); end
        
        % maximum
        data.max{j} = 'N/A';
        try data.max{j} = num2str(motohawk_get_param_ws(blk, 'max_val')); end
        
        % storage type
        data.stortyp{j} = 'volatile';
        try data.stortyp{j} = lower(get_param(blk, 'storage')); end
        if strcmp(data.stortyp{j}, 'display')
            data.stortyp{j} = 'volatile';
        end        
   
        % help text
        data.help{j} = 'N/A';
        try data.help{j} = motohawk_get_param_ws(blk, 'help'); end

        % model path
        data.mdlpath{j} = oneline(get_param(blk, 'Parent'));
        
        % MotoTune group
        data.group{j} = 'N/A';
        try data.group{j} = motohawk_get_param_ws(blk, 'mototune_group'); end
        try data.group{j} = motohawk_get_param_ws(blk, 'group'); end
        data.group{j} = strrep(strrep(data.group{j}, ' | ', '|'), '|', ' | ');
        
    end
    if i == 1
        caldata = data;
    else
        prbdata = data;
    end
end

end


function outtxt = oneline(intxt)
% --- Places text on a single line
nonchar = find(~isstrprop(intxt, 'graphic'));
blank = strfind(intxt, ' ');
for i = 1:length(nonchar)
    if isempty(find(blank == nonchar(i)))
        intxt(nonchar(i)) = ' ';
    end
end
outtxt = intxt;

end